@@ -74,6 +74,7 @@ struct pluto_crypto_worker {
 };
 
 static struct pluto_crypto_req_cont *backlogqueue;
+static struct pluto_crypto_req_cont **backlogqueue_last = &backlogqueue;
 static int                           backlogqueue_len;
 
 static void init_crypto_helper(struct pluto_crypto_worker *w, int n);
@@ -173,7 +174,8 @@ helper_passert_fail(const char *pred_str
 
 void pluto_crypto_helper(int fd, int helpernum)
 {
-    FILE *io = fdopen(fd, "ab+");
+    FILE *in  = fdopen(fd, "rb");
+    FILE *out = fdopen(fd, "wb");
     long reqbuf[PCR_REQ_SIZE/sizeof(long)];
     struct pluto_crypto_req *r;
 
@@ -185,10 +187,10 @@ void pluto_crypto_helper(int fd, int helpernum)
     setpriority(PRIO_PROCESS, 0, 10);
 
     DBG(DBG_CONTROL, DBG_log("helper %d waiting on fd: %d"
-			     , helpernum, fileno(io)));
+			     , helpernum, fileno(in)));
 
     memset(reqbuf, 0, sizeof(reqbuf));
-    while(fread((char*)reqbuf, sizeof(r->pcr_len), 1, io) == 1) {
+    while(fread((char*)reqbuf, sizeof(r->pcr_len), 1, in) == 1) {
 	int restlen;
 	int actnum;
 	unsigned char *reqrest = ((unsigned char *)reqbuf)+sizeof(r->pcr_len);
@@ -197,10 +199,15 @@ void pluto_crypto_helper(int fd, int helpernum)
 	restlen = r->pcr_len-sizeof(r->pcr_len);
 	
 	passert(restlen < (signed)PCR_REQ_SIZE);
+	passert(restlen > 0);
 
-	actnum = fread(reqrest, restlen, 1, io);
+	actnum = fread(reqrest, 1, restlen, in);
 	/* okay, got a basic size, read the rest of it */
-	if(actnum != 1) {
+
+	DBG(DBG_CONTROL, DBG_log("helper %d read %d+4/%d bytesfd: %d"
+				 , helpernum, actnum, r->pcr_len, fileno(in)));
+
+	if(actnum != restlen) {
 	    /* faulty read. die, parent will restart us */
 	    loglog(RC_LOG_SERIOUS, "cryptographic helper(%d) fread(%d)=%d failed: %s\n",
 		   getpid(), restlen, actnum, strerror(errno));
@@ -217,7 +224,8 @@ void pluto_crypto_helper(int fd, int helpernum)
 
 	pluto_do_crypto_op(r);
 
-	actnum = fwrite((unsigned char *)r, r->pcr_len, 1, io);
+	actnum = fwrite((unsigned char *)r, r->pcr_len, 1, out);
+	fflush(out);
 
 	if(actnum != 1) {
 	    loglog(RC_LOG_SERIOUS, "failed to write answer: %d", actnum);
@@ -226,16 +234,53 @@ void pluto_crypto_helper(int fd, int helpernum)
 	memset(reqbuf, 0, sizeof(reqbuf));
     }
 
-    if(!feof(io)) {
-	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", helpernum, strerror(ferror(io)));
+    if(!feof(in)) {
+	loglog(RC_LOG_SERIOUS, "helper %d got error: %s", helpernum, strerror(ferror(in)));
     }
 
     /* probably normal EOF */
-    fclose(io);
+    fclose(in);
+    fclose(out);
     exit(0);
 }
 
 
+/* send the request, make sure it all goes down. */
+static bool crypto_write_request(struct pluto_crypto_worker *w
+				 ,struct pluto_crypto_req *r)
+{
+    unsigned char *wdat = (unsigned char *)r;
+    int wlen = r->pcr_len;
+    int cnt;
+    
+    DBG(DBG_CONTROL
+	, DBG_log("asking helper %d to do %s op on seq: %u (len=%u, pcw_work=%d)"
+		  , w->pcw_helpernum
+		  , enum_show(&pluto_cryptoop_names, r->pcr_type)
+		  , r->pcr_id, r->pcr_len, w->pcw_work+1));
+
+    do {
+	errno=0;
+	cnt = write(w->pcw_pipe, wdat, wlen);
+	
+	if(cnt <= 0) {
+	    openswan_log("write to helper failed: cnt=%d err=%s\n",
+			 cnt, strerror(errno));
+	    return FALSE;
+	}
+	if(DBGP(DBG_CONTROL) || cnt != wlen) {
+	    DBG_log("crypto helper write of request: cnt=%d<wlen=%d. \n", cnt, wlen);
+	}
+
+	wlen -= cnt;
+	wdat += cnt;
+
+    } while(wlen > 0);
+
+    return TRUE;
+}
+
+
 /*
  * this function is called with a request to do some cryptographic operations
  * along with a continuation structure, which will be used to deal with
@@ -284,8 +329,9 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     w = &pc_workers[pc_worker_num];
 
     DBG(DBG_CONTROL
-	, DBG_log("%d: w->pcw_dead: %d w->pcw_work: %d cnt: %d",
-		  pc_worker_num, w->pcw_dead, w->pcw_work, cnt));
+	, DBG_log("%d: w->pcw_dead: %d w->pcw_work: %d cnt: %d %s"
+		  , pc_worker_num, w->pcw_dead, w->pcw_work, cnt
+		  , enum_name(&pluto_cryptoimportance_names,r->pcr_pcim)));
     
     while((w->pcw_dead || (w->pcw_work >= w->pcw_maxbasicwork))
 	  && --cnt > 0) {
@@ -324,13 +370,19 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
 
     if(cnt == 0 && r->pcr_pcim >= pcim_demand_crypto) {
 	/* it is very important. Put it all on a queue for later */
-	cn->pcrc_next = backlogqueue;
-	backlogqueue  = cn;
+	if(!backlogqueue_last) backlogqueue_last = &backlogqueue;
+	*backlogqueue_last = cn;
+	cn->pcrc_next = NULL;
+	backlogqueue_last  = &cn->pcrc_next;
+
+	/* copy the request */
+	r = clone_bytes(r, r->pcr_len, "saved cryptorequest");
+	cn->pcrc_pcr = r;
 	
 	backlogqueue_len++;
 	DBG(DBG_CONTROL
-	    , DBG_log("critical demand crypto operation queued as item %d"
-		      , backlogqueue_len));
+	    , DBG_log("critical demand crypto operation queued on backlog as %d'th item, id: q#%u"
+		      , backlogqueue_len, r->pcr_id));
 	*toomuch = FALSE;
 	return NULL;
     }
@@ -363,18 +415,12 @@ err_t send_crypto_helper_request(struct pluto_crypto_req *r
     passert(w->pcw_pipe != -1);
     passert(w->pcw_work < w->pcw_maxcritwork);
     
-    DBG(DBG_CONTROL
-	, DBG_log("asking helper %d to do %s op on seq: %u"
-		  , w->pcw_helpernum
-		  , enum_show(&pluto_cryptoop_names, r->pcr_type)
-		  , r->pcr_id));
-
-    /* send the request, and then mark the work as having more work */
-    cnt = write(w->pcw_pipe, r, r->pcr_len);
-    if(cnt == -1) {
+    if(!crypto_write_request(w, r)) {
+	openswan_log("failed to write crypto request: %s\n",
+		     strerror(errno));
 	return "failed to write";
-    } 
-
+    }
+	
     w->pcw_work++;
     *toomuch = FALSE;
     return NULL;
@@ -389,8 +435,6 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
     struct pluto_crypto_req_cont *cn;
 
     if(backlogqueue_len > 0) {
-	int cnt;
-
 	passert(backlogqueue != NULL);
 	
 	cn = backlogqueue;
@@ -400,10 +444,10 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
 	r = cn->pcrc_pcr;
       
 	DBG(DBG_CONTROL
-	    , DBG_log("removing backlog item (%d) from queue: %d left"
+	    , DBG_log("removing backlog item id: q#%u from queue: %d left"
 		      , r->pcr_id, backlogqueue_len));
 
-	/* w points to a work. Make sure it is live */
+	/* w points to a worker. Make sure it is live */
 	if(w->pcw_pid == -1) {
 	    init_crypto_helper(w, pc_worker_num);
 	    if(w->pcw_pid == -1) {
@@ -423,19 +467,15 @@ static void crypto_send_backlog(struct pluto_crypto_worker *w)
 	passert(w->pcw_pipe != -1);
 	passert(w->pcw_work > 0);
     
-	DBG(DBG_CONTROL
-	    , DBG_log("asking helper %d to do %s op on seq: %u"
-		      , w->pcw_helpernum
-		      , enum_show(&pluto_cryptoop_names, r->pcr_type)
-		      , r->pcr_id));
-	
-	/* send the request, and then mark the work as having more work */
-	cnt = write(w->pcw_pipe, r, r->pcr_len);
-	if(cnt == -1) {
+	/* send the request, and then mark the worker as having more work */
+	if(!crypto_write_request(w, r)) {
 	    /* XXX invoke callback with failure */
 	    passert(0);
 	    return;
 	} 
+
+	/* if it was on the backlog, it was saved, free it */
+	pfree(r);
 	
 	w->pcw_work++;
     }
@@ -518,12 +558,10 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
     int actlen;
     struct pluto_crypto_req_cont *cn, **cnp;
 
-    /* we can accept more work now that we are about to read from the pipe */
-    w->pcw_work--;
-
-    DBG(DBG_CRYPT, DBG_log("helper %u has work (cnt now %d)"
-			   ,w->pcw_helpernum
-			   ,w->pcw_work));
+    DBG(DBG_CRYPT|DBG_CONTROL
+	, DBG_log("helper %u has finished work (cnt now %d)"
+		  ,w->pcw_helpernum
+		  ,w->pcw_work));
 
     /* read from the pipe */
     actlen = read(w->pcw_pipe, (char *)reqbuf, sizeof(r->pcr_len));
@@ -543,6 +581,9 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
 	return;
     }
 
+    /* we can accept more work now that we have read from the pipe */
+    w->pcw_work--;
+
     r = (struct pluto_crypto_req *)reqbuf;
 
     if(r->pcr_len > sizeof(reqbuf)) {
@@ -578,9 +619,9 @@ void handle_helper_comm(struct pluto_crypto_worker *w)
 	inloc   += actlen;
     }
 
-    DBG(DBG_CRYPT, DBG_log("helper %u replies to sequence %u"
-			   ,w->pcw_helpernum
-			   ,r->pcr_id));
+    DBG(DBG_CRYPT|DBG_CONTROL, DBG_log("helper %u replies to id: q#%u"
+				       ,w->pcw_helpernum
+				       ,r->pcr_id));
 
     /*
      * if there is work queued, then send it off after reading, since this
